<?php

namespace QUI\CSRF;

use Exception;
use QUI;
use ParagonIE\AntiCSRF\AntiCSRF;

use function defined;
use function htmlspecialchars;
use function md5;
use function rand;

use const ENT_QUOTES;

class EventHandler
{
    public static function isActive(): bool
    {
        try {
            $Config = QUI::getPackage('quiqqer/csrf')->getConfig();
            $active = $Config?->getValue('general_csrf', 'active');
        } catch (QUI\Exception) {
            return false;
        }

        if (empty($active)) {
            return false;
        }

        return true;
    }

    public static function onTemplateGetHeader(QUI\Template $TemplateManager): void
    {
        if (!self::isActive()) {
            return;
        }

        $token = md5((string)rand(100000, 999999));
        $index = md5((string)rand(100000, 999999));

        try {
            $csrf = new AntiCSRF();
            $tokenArray = $csrf->getTokenArray();

            if (isset($tokenArray['_CSRF_TOKEN'])) {
                $token = $tokenArray['_CSRF_TOKEN'];
                $token = htmlspecialchars($token, ENT_QUOTES, 'UTF-8');
            }

            if (isset($tokenArray['_CSRF_INDEX'])) {
                $index = $tokenArray['_CSRF_INDEX'];
                $index = htmlspecialchars($index, ENT_QUOTES, 'UTF-8');
            }
        } catch (\Exception $Exception) {
            QUI\System\Log::addError($Exception->getMessage(), [
                'type' => 'csrf token creation'
            ]);
        }

        $TemplateManager->extendHeader(
            '
            <meta content="' . $token . '" name="csrf-token" data-no-cache="1" />
            <meta content="' . $index . '" name="csrf-index" data-no-cache="1" />
            <script src="' . URL_OPT_DIR . 'quiqqer/csrf/bin/csrf.js"></script>
            '
        );
    }

    public static function onOutputParseEnd(string &$result): void
    {
        if (!self::isActive()) {
            return;
        }

        $result = preg_replace_callback(
            '/<form.*?<\/form>/ms',
            function ($matches) {
                $form = $matches[0];
                $csrf = new AntiCSRF();

                try {
                    $form = str_replace(
                        '</form>',
                        $csrf->insertToken('', false) . '</form>',
                        $form
                    );
                } catch (Exception) {
                }

                return $form;
            },
            $result
        );
    }

    public static function onHeaderLoaded(): void
    {
        if (!self::isActive()) {
            return;
        }

        if (php_sapi_name() !== 'cli') {
            return;
        }

        if (defined('QUIQQER_CONSOLE') && QUIQQER_CONSOLE) {
            return;
        }

        try {
            $csrf = new AntiCSRF();
        } catch (\Error $e) {
            QUI\System\Log::addError('CSRF token error ' . $e->getMessage());
            return;
        }

        if (defined('QUIQQER_AJAX')) {
            return;
        }

        if (!empty($_POST)) {
            $csrf->validateRequestOrThrow();
        }
    }
}
